import mtapi.mt5.*;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;

public class Main {

    public static void main(String[] args) throws Exception {
        new Main().run();
    }

    public void run() throws Exception {
        //Logger.OnMsg.addListener(new OnMsg());
        //quotes();
        //orderHistory();
        marketOrder();
        //marketOrderAsync();
        //modifyOrder();
        //pendingOrder();
        //orderHistory1();
        //serversDat();
        //accountBalanceWithServersDat();
        //symbols();
    }

    private void symbols() throws Exception {
        MT5API api = new MT5API(8631960, "dd4444vv", "mt5demo.acetopfx.com", 443);
        System.out.println("Connecting...");
        api.Connect();
        System.out.println("Connected");
        for (SymbolInfo symbol : api.Symbols.Infos)
            System.out.println(symbol.Name);
        System.out.println(api.AccountMargin());
    }

    private void accountBalanceWithServersDat() throws IOException {
        Map<String, String[]>  servers = serversDat();
        String serverName = "Acetop-Demo";
        String[] addresses = servers.get(serverName);
        for(String address : addresses) {
            try {
                Map.Entry<String, Integer> hostAndPort = parseAdr(address);
                MT5API api = new MT5API(8631960, "dd4444vv", hostAndPort.getKey(), hostAndPort.getValue());
                api.Connect();
                System.out.println(api.AccountBalance());
                break;
            } catch (Exception e) {
                continue;
            }
        }
    }

    Map<String, String[]> serversDat() throws IOException {
        Server[] servers = MT5API.LoadServersDat("C:\\Users\\User\\AppData\\Roaming\\MetaQuotes\\Terminal\\F70BE0C3730B4029EDFAC1A0EED9A816\\config\\servers.dat");
        Map<String, String[]> res = new HashMap<>();
        for (Server server : servers) {
            String name = null;
            if (server.ServerInfo != null)
            {
                System.out.println(server.ServerInfo.ServerName);
                name = server.ServerInfo.ServerName;
            }
            if (server.ServerInfoEx != null) {
                System.out.println(server.ServerInfoEx.ServerName);
                name = server.ServerInfoEx.ServerName;
            }
            if(name == null)
                throw new RuntimeException("Server name is null");
            List<String> adr = new LinkedList<>();
            for (Access access : server.Accesses)
                for (AddressRec addr : access.Addresses) {
                    System.out.println(addr.Address);
                    adr.add(addr.Address);
                }
            res.put(name, adr.toArray(new String[0]));
            System.out.println();
        }
        return res;
    }

    Map.Entry<String, Integer> parseAdr(String ip)
    {
        int port;
        int i = 0;
        String host = "";
        while (i < ip.length() && ip.getBytes()[i] != (byte)':')
            host += (char)ip.getBytes()[i++];
        if (i == ip.length())
            port = 443;
        else
        {
            i++;
            String strPort = "";
            while (i < ip.length())
                strPort += (char)ip.getBytes()[i++];
            port = Integer.parseInt(strPort);
        }
        return new AbstractMap.SimpleEntry<String, Integer>(host.trim(), port);
    }



    private void orderHistory1() throws Exception {
        //MT5API api = new MT5API(99083840, "ychqluy5", "112.126.96.179", 443);
        MT5API api = new MT5API(35388558, "oxaszz8p", "47.106.10.16", 1964);
        api.Connect();
        System.out.println("Connected. Balance = " + api.AccountBalance());
        api.OnOrderHistory.addListener(new OnOrderHistory() {
            @Override
            public void invoke(MT5API sender, Order[] orders, DealInternal[] internal_deals, OrderInternal[] internal_orders) {
                for (Order order: orders) {
                    System.out.println(order.Ticket);
                }
            }
        });
        //api.RequestOrderHistory(LocalDateTime.now().minusDays(100),LocalDateTime.now().plusDays(1) );
        api.RequestOrderHistory(LocalDateTime.now().minusDays(10),LocalDateTime.now().plusDays(1) );
    }

    void pendingOrder() throws Exception {
        MT5API api = new MT5API(8631960, "dd4444vv", "mt5demo.acetopfx.com", 443);
        System.out.println("Connecting...");
        api.Connect();
        System.out.println(api.AccountMargin());
        System.out.println("Connected");
        String symbol = "EURUSD.DEMO";
        api.Subscribe(symbol);
        while (api.GetQuote(symbol) == null)
            Thread.sleep(1);
        Order o = api.OrderSend(symbol, 0.02, api.GetQuote(symbol).Ask*1.1, OrderType.BuyStop, 0, 0, 100, null, 0, FillPolicy.FlashFill, 0, LocalDateTime.now().plusDays(3));
        System.out.println("Order " + o.Ticket + " opened at " + o.OpenPrice);
        System.out.println("Opened orders:");
        for (Order order: api.GetOpenedOrders()) {
            System.out.println(order.Ticket);
        }
//        o = api.OrderClose(o.Ticket, o.Symbol, o.OpenPrice, o.Lots, o.OrderType, 0);
//        System.out.println("Closed");
//        System.out.println("Opened orders:");
//        for (Order order: api.GetOpenedOrders()) {
//            System.out.println(order.Ticket);
//        }
    }

    void modifyOrder() throws Exception {
        MT5API api = new MT5API(8631960, "dd4444vv", "mt5demo.acetopfx.com", 443);
        System.out.println("Connecting...");
        api.Connect();
        System.out.println("Connected");
        String symbol = "EURUSD.DEMO";
        api.Subscribe(symbol);
        while (api.GetQuote(symbol) == null)
            Thread.sleep(1);
        Order o = api.OrderSend(symbol, 0.02, api.GetQuote(symbol).Ask, OrderType.Buy, 0, 0, 100, null, 0);
        System.out.println("Order " + o.Ticket + " opened at " + o.OpenPrice);
        api.OrderModify(o.Ticket, o.Symbol, o.Lots, o.OpenPrice, OrderType.Buy, 0.9, 1.2, 0, null, 0);
        System.out.println("Modified");
        for (Order order: api.GetOpenedOrders()) {
            System.out.println(order.Ticket);
        }
        o = api.OrderClose(o.Ticket, o.Symbol, api.GetQuote(symbol).Bid, o.Lots, o.OrderType, 100);
        System.out.println("Closed at " + o.ClosePrice);
    }

    private void marketOrderAsync() throws Exception {
        MT5API api = new MT5API(8631960, "dd4444vv", "mt5demo.acetopfx.com", 443);
        api.Connect();
        System.out.println("Connected. Balance = " + api.AccountBalance());
        String symbol = "EURUSD.DEMO";
        api.Subscribe(symbol);
        while (api.GetQuote(symbol) == null)
            Thread.sleep(1);
        System.out.println("Got quote");
        api.OnOrderProgress.addListener(new OnOrderProgress() {
            @Override
            public void invoke(MT5API sender, OrderProgress progress) {
                System.out.println(progress.TradeRequest.RequestId + " " + progress.TradeResult.Status);
            }
        });
        int id  = api.GetRequestId();
        api.OrderSendAsync(id, symbol, 0.01, api.GetQuote(symbol).Ask, OrderType.Buy, 0, 0, 1000, null, 0);
    }

    private void marketOrder() throws Exception {
        MT5API api = new MT5API(8631960, "dd4444vv", "mt5demo.acetopfx.com", 443);
        // MT5API api = new MT5API(5025145, "zYphUSCh", "51.140.72.12", 443);
        // MT5API api = new MT5API(245144, "vlyqcpo5", "148.251.41.107", 443);
        api.Connect();
        api.PlacedType = PlacedType.Manually;
        System.out.println("Connected. Balance = " + api.AccountBalance());
        String symbol = "EURUSD.DEMO";
        //api.Subscribe(symbol);
        //while (api.GetQuote(symbol) == null)
        //    Thread.sleep(1);
        //System.out.println("Got quote");
        Order o = api.OrderSend(symbol, 0.01, 0, OrderType.Sell, 0, 0, 100, null, 0);

        System.out.println("Order " + o.Ticket + " opened at " + o.OpenPrice);
        System.out.println("Opened orders:");
        for (Order order : api.GetOpenedOrders()) {
            System.out.println(order.Ticket);
        }
        o = api.OrderClose(o.Ticket, o.Symbol, 0, o.Lots, o.OrderType, 100);
        System.out.println("Closed at " + o.ClosePrice);
        System.out.println("Opened orders:");
        for (Order order : api.GetOpenedOrders()) {
            System.out.println(order.Ticket);
        }
    }

    private void orderHistory() throws Exception {
        MT5API api = new MT5API(99083840, "ychqluy5", "112.126.96.179", 443);
        api.Connect();

        System.out.println("Connected. Balance = " + api.AccountBalance());
        api.OnOrderHistory.addListener(new OnOrderHistory() {
            @Override
            public void invoke(MT5API sender, Order[] orders, DealInternal[] internal_deals, OrderInternal[] internal_orders) {
                for (Order order: orders) {
                    System.out.println(order.Ticket);
                }
            }
        });
        api.RequestOrderHistory(LocalDateTime.now().minusDays(100),LocalDateTime.now().plusDays(1) );
    }

    public void quotes() throws Exception {
        MT5API api = new MT5API(99083840, "ychqluy5", "112.126.96.179", 443);
        api.Connect();
        System.out.println("Connected. Balance = " + api.AccountBalance());
        api.Subscribe("EURUSD");
        api.OnQuote.addListener(new OnQuote() {
            @Override
            public void invoke(MT5API sender, Quote quote) {
                System.out.println(quote);
            }
        });
    }

    class OnMsg implements OnMsgHandler
    {
        @Override
        public void invoke(Object sender, String msg, MsgType type) {
            if(type == MsgType.Trace)
                return;
            String m = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(new Date()) + " " + type.toString().substring(0, 3) + ": "  + sender.getClass().getSimpleName();
            m+= " " + msg;
            System.out.println(m);
        }
    }
}
